import os
import cv2
import time
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
from concurrent.futures import ThreadPoolExecutor

class Watcher:
    DIRECTORY_TO_WATCH = "Z:/nginxFileServe/PhotoGallery"  # Directory to monitor

    def __init__(self):
        self.observer = Observer()
        self.executor = ThreadPoolExecutor(max_workers=4)  # ThreadPoolExecutor for parallel processing

    def run(self):
        process_existing_files(self.DIRECTORY_TO_WATCH)
        event_handler = Handler(self.executor)
        self.observer.schedule(event_handler, self.DIRECTORY_TO_WATCH, recursive=True)
        self.observer.start()
        try:
            while True:
                time.sleep(10)
        except KeyboardInterrupt:
            self.observer.stop()
            print("Observer Stopped")
        self.observer.join()

class Handler(FileSystemEventHandler):
    def __init__(self, executor):
        self.executor = executor

    def on_any_event(self, event):
        if event.is_directory or 'thumbs' in os.path.split(event.src_path)[0]:
            return None
        elif event.event_type == 'created' and event.src_path.lower().endswith(('.mp4', '.png', '.jpg', '.jpeg')):
            print(f"Received created event - {event.src_path}.")
            thumbs_dir = self.get_thumbs_dir(event.src_path)
            if thumbs_dir:
                os.makedirs(thumbs_dir, exist_ok=True)
                self.executor.submit(self.process_file, event.src_path, thumbs_dir)

    def get_thumbs_dir(self, file_path):
        parent_dir = os.path.dirname(file_path)
        if 'thumbs' in parent_dir.split(os.sep):
            return None
        return os.path.join(parent_dir, 'thumbs')
    
    @staticmethod
    def process_file(file_path, thumbs_dir):
        thumb_file_ext = '.mp4' if file_path.lower().endswith('.mp4') else '.jpg'
        thumb_file_name = os.path.splitext(os.path.basename(file_path))[0] + thumb_file_ext
        thumb_file_path = os.path.join(thumbs_dir, thumb_file_name)
        if os.path.exists(thumb_file_path):
            print(f"Thumbnail already exists: {thumb_file_path}")
            return

        if file_path.lower().endswith('.mp4'):
            Handler.extract_frame(file_path, thumb_file_path)
        elif file_path.lower().endswith(('.png', '.jpg', '.jpeg')):
            Handler.resize_image(file_path, thumb_file_path)

    @staticmethod
    def extract_frame(video_path, thumb_file_path):
        try:
            cap = cv2.VideoCapture(video_path)
            success, image = cap.read()
            if success:
                jpg_thumb_path = thumb_file_path.replace('.mp4', '.jpg')
                Handler.add_play_button(image, jpg_thumb_path)
                if os.path.exists(jpg_thumb_path):
                    os.rename(jpg_thumb_path, thumb_file_path)
            cap.release()
        except Exception as e:
            print(f"Error processing video {video_path}: {e}")

    @staticmethod
    def add_play_button(image, thumb_file_path):
        try:
            play_button_image = cv2.imread('Z:/nginxFileServe/trans.png')  # Path to your play button image
            if play_button_image is None:
                raise FileNotFoundError("Play button image not found.")

            play_button_size = 400
            play_button_image = cv2.resize(play_button_image, (play_button_size, play_button_size))
            h, w = image.shape[:2]
            x, y = (w - play_button_size) // 2, (h - play_button_size) // 2
            image[y:y+play_button_size, x:x+play_button_size] = play_button_image

            cv2.imwrite(thumb_file_path, image)
            print(f"Thumbnail saved at {thumb_file_path}")
        except Exception as e:
            print(f"Error adding play button to image: {e}")

    @staticmethod
    def resize_image(image_path, thumb_file_path):
        try:
            img = cv2.imread(image_path)
            if img is None or img.size == 0:
                raise ValueError("Unable to read image or image is empty")

            height, width = img.shape[:2]
            if height < 2 or width < 2:
                raise ValueError("Image dimensions too small to resize")

            resized_img = cv2.resize(img, (width // 2, height // 2))
            cv2.imwrite(thumb_file_path, resized_img)
            print(f"Resized image saved at {thumb_file_path}")
        except Exception as e:
            print(f"Error processing image {image_path}: {e}")

def process_existing_files(directory):
    for root, dirs, files in os.walk(directory):
        if 'thumbs' in root.split(os.sep):
            continue
        for file in files:
            if file.lower().endswith(('.mp4', '.png', '.jpg', '.jpeg')):
                file_path = os.path.join(root, file)
                thumbs_dir = os.path.join(root, 'thumbs')
                os.makedirs(thumbs_dir, exist_ok=True)
                print(f"Processing existing file: {file_path}")
                Handler.process_file(file_path, thumbs_dir)

if __name__ == '__main__':
    w = Watcher()
    w.run()
